local _,stash = ...
local L = stash.localization
local C = stash.constants
stash.tooltips = {}

local gameTooltip = StashGameTooltip
local moneyTooltip = StashMoneyTooltip

local suppressTooltip = false

-- hides all tooltips
function stash.tooltips:HideTooltip()
    gameTooltip:Hide()
    moneyTooltip:Hide()
    stash.timer:Stop(self.AddLocationsToTooltip)
end

-- when something is likely to show a tooltip and we don't want it shown (such as changing from moneypanel to itemspanel
-- while mouse was over okay button but is now over moneybutton), call this to prevent it from being shown for a short time
function stash.tooltips:Pause()
    suppressTooltip = true
    stash.timer:Start(C.PAUSE_TOOLTIP_TIMER,stash.tooltips.Resume,stash.tooltips)
end

-- called auto
function stash.tooltips:Resume()
    suppressTooltip = false
end

-- for simple tooltips like the minimap button tooltip
function stash.tooltips:ShowSimpleTooltip(parent,anchor,title,body)
    gameTooltip:SetOwner(parent,anchor or "ANCHOR_LEFT")
    gameTooltip:AddLine(title)
    gameTooltip:AddLine(body)
    gameTooltip:Show()
end

-- called in the onenter of an item list button, displays a tooltip of the item and appends who has the item where
function stash.tooltips:ShowItemTooltip(button,itemID)
    if suppressTooltip then
        return
    end
    local sx,sy = button:GetCenter()
    local ss = button:GetEffectiveScale()
    local ux,uy = UIParent:GetCenter()
    local us = UIParent:GetEffectiveScale()
    gameTooltip:SetOwner(button,sx*ss<ux*us and "ANCHOR_RIGHT" or "ANCHOR_LEFT")
    if type(itemID)=="number" then
        gameTooltip:SetItemByID(itemID)
    elseif type(itemID)=="string" and itemID:match("^battlepet") then
        self:SetBattlePetTooltip(itemID)
    elseif type(itemID)=="string" and itemID:match("^currency") then
        self:SetCurrencyTooltip(itemID)
    else
        return -- don't know what this is, don't show a tooltip
    end
    gameTooltip.itemID = itemID
    gameTooltip:Show()
    stash.timer:Start(C.TOOLTIP_LOCATION_DELAY,self.AddLocationsToTooltip,self)
end

function stash.tooltips:ShowMoneyTooltip(button)
    if suppressTooltip then
        return
    end
    local money = stash.info:FindMoney()
    if not moneyTooltip.lines then
        moneyTooltip.lines = {}
    end
    local maxWidth = 0 -- becomes width of widest button
    local numLines = 0 -- number of lines added to tooltip
    local hasMoneyElsewhere = false -- becomes true if there's money in other realms/factions
    -- set all lines
    for i=1,#money do
        -- but only for current realm and faction (rest can go in list)
        if money[i][C.FOUND_REALM]==stash.current.realm and money[i][C.FOUND_FACTION]==stash.current.faction then
            if not moneyTooltip.lines[i] then
                moneyTooltip.lines[i] = CreateFrame("Frame",nil,moneyTooltip,"StashMoneyTooltipLineTemplate")
                moneyTooltip.lines[i]:SetPoint("TOPLEFT",8,-6-(i-1)*18)
            end
            local color = money[i][C.FOUND_PLAYER]:match(" ") and C.HEX_GREEN or C.FACTION_COLORS[money[i][C.FOUND_FACTION]] or C.HEX_NEUTRAL
            moneyTooltip.lines[i].Name:SetText(format("%s%s",color,money[i][C.FOUND_PLAYER]))
            moneyTooltip.lines[i].Money:SetAmount(money[i][C.FOUND_COUNT])
            moneyTooltip.lines[i].Stripe:SetShown(i%2==0)
            maxWidth = max(maxWidth,moneyTooltip.lines[i].Name:GetStringWidth()+moneyTooltip.lines[i].Money:GetWidth()+8)
            numLines = numLines + 1
        else
            hasMoneyElsewhere = true
        end
    end

    -- add a line that says "Click for all realms/factions" if there's money on other realms or factions
    if hasMoneyElsewhere then
        moneyTooltip.Instructions:SetText(format(L["%s Click for all realms/factions"],C.LMB))
        maxWidth = max(maxWidth,moneyTooltip.Instructions:GetStringWidth())
        moneyTooltip.Instructions:Show()
    else
        moneyTooltip.Instructions:Hide()
    end

    -- resize all lines to max width
    for i=1,numLines do
        moneyTooltip.lines[i]:SetWidth(maxWidth)
        moneyTooltip.lines[i]:Show()
    end
    -- hide any leftover lines (characters reset or filters changed)
    for i=numLines+1,#moneyTooltip.lines do
        moneyTooltip.lines[i]:Hide()
    end
    moneyTooltip:SetSize(maxWidth+16,numLines*18+12+(hasMoneyElsewhere and 18 or 0))
    moneyTooltip:SetPoint("BOTTOMRIGHT",button,"TOPRIGHT",0,0)
    moneyTooltip:SetBackdropColor(0.075,0.075,0.075)
    --moneyTooltip:SetBackdropBorderColor(0.5,0.5,0.5)    
    moneyTooltip:Show()
    moneyTooltip.numLines = numLines
end

-- for caged pets, not using the default UI's BattlePetTooltip since the stats are unknown. this adds some
-- info to gameTooltip about the caged pet such as name, type, collected count and source
function stash.tooltips:SetBattlePetTooltip(itemID)
    if suppressTooltip then
        return
    end
    local speciesID = tonumber((itemID or ""):match("battlepet:(%d+)"))
    if speciesID then
        local name,icon,petType,_,source,_,_,canBattle = C_PetJournal.GetPetInfoBySpeciesID(speciesID)
        local count,maxCount = C_PetJournal.GetNumCollectedInfo(speciesID)
        if name then
            gameTooltip:AddLine(C.HEX_WHITE..name)
            gameTooltip:AddLine(C.HEX_BLUE..(canBattle and L["Caged Battle Pet"] or L["Caged Companion Pet"]))
            gameTooltip:AddLine(C.HEX_GREEN.._G["BATTLE_PET_NAME_"..petType])
            gameTooltip:AddLine(((not count or count==0) and C.HEX_WHITE or count==maxCount and C.HEX_RED or C.HEX_GREEN)..format(ITEM_PET_KNOWN,count,maxCount))
            gameTooltip:AddLine(" ")
            gameTooltip:AddLine(C.HEX_WHITE..source)
        end
    end
end

-- adds the locations that the itemID in the current gameTooltip can be found to the tooltip
function stash.tooltips:AddLocationsToTooltip()
    stash.tooltips:AddLocationsToAnyTooltip(gameTooltip,gameTooltip.itemID,true)
end

-- adds the locations that the itemID can be found to the given tooltip; addSpace to include a space before the locations
function stash.tooltips:AddLocationsToAnyTooltip(tooltip,itemID,addSpace)
    if itemID then
        -- adding who has the item and where to the tooltip
        local found = stash.info:FindItem(itemID)
        if found and #found>0 then
            local spacerAdded = not addSpace -- on first line added, add a space first
            local currentRealm = stash.current.realm -- locations are always sorted current realm first
            local foundRealm = currentRealm -- for watching when realm changes in tooltip
            for _,foundInfo in ipairs(found) do
                local locationInfo = foundInfo[C.FOUND_LOCATION] and C.LOCATIONS[foundInfo[C.FOUND_LOCATION]]
                if locationInfo then
                    local phrase = locationInfo.phrase
                    if phrase then
                        if not spacerAdded then
                            tooltip:AddLine(" ")
                            spacerAdded = true
                        end
                        if foundInfo[C.FOUND_REALM]~=foundRealm then
                            foundRealm = foundInfo[C.FOUND_REALM]
                            tooltip:AddLine(format(L["%sOn %s:"],C.HEX_GREEN,foundRealm))
                        end
                        local locationIcon = C.LOCATIONS[foundInfo[C.FOUND_LOCATION]] and C.LOCATIONS[foundInfo[C.FOUND_LOCATION]].icon or C.UNKNOWN_ICON
                        local factionColor = C.FACTION_COLORS[foundInfo[C.FOUND_FACTION]] or C.TOOLTIP_LOCATION_COLOR
                        --tooltip:AddLine(format(locationIcon.." "..C.TOOLTIP_LOCATION_COLOR..phrase,factionColor,foundInfo[C.FOUND_COUNT],foundInfo[C.FOUND_PLAYER]))
                        tooltip:AddLine(format(phrase,locationIcon,factionColor,BreakUpLargeNumbers(foundInfo[C.FOUND_COUNT]),foundInfo[C.FOUND_PLAYER]))
                    end
                end
            end
        end
        tooltip:Show()        
    end
end

function stash.tooltips:SetCurrencyTooltip(itemID)
    local currencyID = tonumber(itemID:match("(%d+)"))
    if not currencyID then
        return
    end
    local info = C_CurrencyInfo.GetBasicCurrencyInfo(currencyID)
    if info then
        local color = C.QUALITY_COLORS[info.quality or 1] or C.QUALITY_COLORS[1]
        gameTooltip:AddLine(color.hex..info.name)
        gameTooltip:AddLine(C.HEX_BLUE..L["Currency"])
        gameTooltip:AddLine(info.description,1,0.82,0,true)
        local extraInfo = C_CurrencyInfo.GetCurrencyInfo(currencyID)
        if extraInfo.maxQuantity and extraInfo.maxQuantity>0 then
            gameTooltip:AddLine(" ")
            gameTooltip:AddLine(format(L["Maximum %s%s\124r per character"],C.HEX_WHITE,BreakUpLargeNumbers(extraInfo.maxQuantity)))
        end

    end
end
